/*
 * jPSXdec: PlayStation 1 Media Decoder/Converter in Java
 * Copyright (C) 2007-2020  Michael Sabin
 * All rights reserved.
 *
 * Redistribution and use of the jPSXdec code or any derivative works are
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions may not be sold, nor may they be used in commercial
 *    or revenue-generating business activities.
 *
 *  * Redistributions that are modified from the original source must
 *    include the complete source code, including the source code for all
 *    components used by a binary built from the modified sources. However, as
 *    a special exception, the source code distributed need not include
 *    anything that is normally distributed (in either source or binary form)
 *    with the major components (compiler, kernel, and so on) of the operating
 *    system on which the executable runs, unless that component itself
 *    accompanies the executable.
 *
 *  * Redistributions must reproduce the above copyright notice, this list
 *    of conditions and the following disclaimer in the documentation and/or
 *    other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package jpsxdec.modules.sharedaudio;

import javax.annotation.Nonnull;
import javax.sound.sampled.AudioFormat;
import jpsxdec.modules.SectorClaimSystem;

/** Interface for decoders generated by a DiscItem.
 *  Hides the decoder implementation, but also allows it to be passed around. */
public interface ISectorAudioDecoder {

    /** Must be set before using this class. */
    void setAudioListener(@Nonnull DecodedAudioPacket.Listener listener);

    /** The format of the audio data that will be fed to the
     * {@link DecodedAudioPacket.Listener} listener. */
    @Nonnull AudioFormat getOutputFormat();

    void attachToSectorClaimer(@Nonnull SectorClaimSystem scs);

    double getVolume();
    
    /** Sector where the audio begins to play. */
    int getAbsolutePresentationStartSector();

    /** Physical start sector, not presentation start sector. */
    int getStartSector();
    /** Physical end sector, not presentation end sector. */
    int getEndSector();

    int getSampleFramesPerSecond();

    /** @return -1, 1 or 2 */
    int getDiscSpeed();
}
